/*
* Level 0 variables defined for all objects.
*/
#define jg_gregorian_start_date_id 1
#define jg_gregorian_stop_date_id 2
#define jg_julian_start_date_id 6
#define jg_julian_stop_date_id 7
#define jg_mooring_id 0
#define jg_lat_id 3
#define jg_lon_id 4
#define jg_depth_id 5
#define jg_time_id 8


/*
* Variable structure definition.
*
*/
typedef struct _Gom_Var {
	char 		name[100];		/* variable name */
	int			level;			/* level number of variable */
	nc_type		data_type;		/* netcdf data type */
	char		value[100];		/* contains string representation 
								 * of value
								 */
	int			valid;			/* whether or not this
								   particular variable is valid
								   in this object.  0 for no,
								   1 for yes. */
								
} Gom_Var;
 

/*
* gom object definition
*/
typedef struct _gom_obj {
	
	FILE	*header_fp;		/* header file pointer */
	char	ncfilename[100];/* name of current netcdf file */
	int		ncid;			/* id of current netcdf file */

	/*char 	comment[100][1000];	*/		/*object comments */
	char	*comment[100];
	int   	num_comments,		 	
			num_level0_comments,	 
			num_level1_comments,	/* total number of comments
									 * should equal level 0
									 * comments + level 1
									 * comments.
									 */
			level_0_comments_valid,
			level_1_comments_valid, /*
									 * flags set to indicate
									 * what level of comments
									 * are to be returned.
									 */
			current_comment;	/* index of current comment */

	Gom_Var var[100];
	int 	num_vars,		/* total number of variables */
			num0v,			/* number of level 0 variables */
			num1v;			/* number of level 1 variables */
} Gom_Obj;

